// Disk
// tULAMIDE
// PS 2.0
// Focus on a part of the object with an oval shape.

//#PARAM percent size 0.65 : Size : The size of the lens.
//#PARAM percent centerX 0.5 : Horizontal Center : The horizontal center of the lens.
//#PARAM percent centerY 0.5 : Vertical Center : The vertical center of the lens.
//#PARAM percent smooth 0.25 : Smooth Border : How much of the lens size is used to smooth the border.
//#PARAM percent darken 0.75 : Darken : How much the area outside the lens is darkened.
//#PARAM percent opacity 1.0 : Opacity : The opacity outside the lens.
//#PARAM float replace 0.0 : Replace Content : To completely replace the content of the object (0 = replace off).

float size;
float centerX;
float centerY;
float smooth;
float darken;
float opacity;
float replace;

float boxLeft;
float boxTop;
float boxWidth;
float boxHeight;

// Foreground texture
texture ForegroundTexture;

// Foreground sampler
sampler2D foreground = sampler_state {
    Texture = (ForegroundTexture);
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
};

//Shaping 
float4 lens(float dark, float opa, float grad, float4 ocolor)
{	
    ocolor.rgb /= ocolor.a;
    ocolor.rgb *= 1.0 - (dark * grad);
    ocolor.a *= 1.0 - (opa * grad);
    ocolor.rgb *= ocolor.a;

    return ocolor;
}

//Replacing
float4 transparentlens(float dark, float opa, float grad)
{
    float chval = 1.0 - dark;
    float4 tcolor = float4(chval, chval, chval, 1.0);
    tcolor.rgb *= (opa) * grad;
    tcolor.a = (opa) * grad;

    return tcolor;
}

// Effect function
float4 EffectProcess( float2 Tex : TEXCOORD0 ) : COLOR0
{
    float2 coord = float2((Tex.x - boxLeft) / boxWidth, (Tex.y - boxTop) / boxHeight);
    float2 center = float2(centerX, centerY);
    float dist = distance(coord, center);
    float rsize = size * 0.5;
    float ssize = rsize * smooth;
    float mnsize = rsize - ssize;
    float perc = saturate( (dist - mnsize) / ssize );

    float4 color = tex2D( foreground, Tex.xy );
    float4 output = tex2D( foreground, Tex.xy );
    if(replace != 0)
        output = transparentlens(darken, opacity, perc);
    else
        output = lens(darken, 1.0 - opacity, perc, color);

    return output;
}

// ConstructEffect
technique ConstructEffect
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 EffectProcess();
    }
}